@@ -23,7 +23,8 @@ from .norms import (
   norm_data,
   b64_nopad,
   hash_sha256,
-  email_encode_items )
+  email_encode_items,
+  TimeEncode )
 
 from .pep import (
   CompatibilityTags,
@@ -49,6 +50,14 @@ from .pep import (
   norm_entry_point_name,
   norm_entry_point_ref )
 
+from .path import (
+  PatternError,
+  PathMatcher,
+  PathFilter,
+  partition,
+  combine_ignore_patterns,
+  contains )
+
 from .dist_file import (
   dist_base,
   dist_zip,
@@ -57,7 +66,6 @@ from .dist_file import (
   dist_source_targz,
   dist_binary_wheel,
   FileOutsideRootError,
-  root_ignore_patterns,
   dist_iter,
   dist_copy )
 
@@ -67,6 +75,9 @@ from .pkginfo import (
   PkgInfoURL,
   PkgInfo )
 
+from .builder import (
+  Builder )
+
 from .pyproj import (
   EntryPointError,
   PyProjBase )
