@@ -113,6 +113,10 @@ class ValidationError( ValueError ):
   def __repr__(self):
     return str(self)
 
+#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+class RequiredValueError( ValidationError ):
+  pass
+
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class ValidDefinitionError( ValidationError ):
   pass
@@ -199,41 +203,48 @@ class Special:
 
   #-----------------------------------------------------------------------------
   def __eq__(self, other):
-    return str(self) == str(other)
+    return type(self) is type(other)
+
+  #-----------------------------------------------------------------------------
+  def __ne__(self, other):
+    return type(self) is not type(other)
 
   #-----------------------------------------------------------------------------
   def __hash__(self):
     return hash(str(self))
 
+#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class Optional(Special):
   """Optional value
   """
   pass
 
+#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class Required(Special):
   """Required value
   """
   pass
 
+#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class NotSet(Special):
   """Special value indicating a value is not set
   """
   pass
 
-optional = Optional()
-required = Required()
-notset = NotSet()
+OPTIONAL = Optional()
+REQUIRED = Required()
+NOTSET = NotSet()
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 def validate(val, default, validators):
   """Internal method to apply default value and validators
   """
   if val is None:
-    if default is optional:
+    if OPTIONAL == default:
       return None
 
-    elif default is required:
-      raise ValidationError(f"Value is required")
+    elif REQUIRED == default:
+      raise RequiredValueError(f"Value is required")
 
     else:
       val = default
@@ -265,7 +276,7 @@ def validate(val, default, validators):
 
       for _validator in validator:
         try:
-          val = validate(val, required, _validator)
+          val = validate(val, REQUIRED, _validator)
           break
         except Exception as e:
           errs.append((_validator, e))
@@ -324,18 +335,19 @@ class Validator:
   """Validates a value
   """
   #-----------------------------------------------------------------------------
-  def __init__(self, *args, default = notset):
+  def __init__(self, *args, default = NOTSET):
 
     args = list(args)
 
-    if default is notset:
-      default = required
+    # TODO: change comparisons from 'is' to "==", reimplement Special __eq__
+    if NOTSET == default:
+      default = REQUIRED
 
       if len(args):
         v = args.pop(0)
 
-        if v in [ None, optional ]:
-          default = optional
+        if OPTIONAL == v or v is None:
+          default = OPTIONAL
 
         elif any(isinstance(v, t) for t in [bool, int, float, str, Sequence, Mapping]):
           default = v
@@ -346,6 +358,10 @@ class Validator:
 
           try:
             default = v()
+          except RequiredValueError:
+            # this really is a required value
+            pass
+
           except Exception as e:
             # the type cannot be instantiated without arguments
             raise ValidDefinitionError(
@@ -356,9 +372,9 @@ class Validator:
           args.insert(0, v)
 
     if default is None:
-      default = optional
+      default = OPTIONAL
 
-    if len(args) == 0 and default not in [required, optional]:
+    if len(args) == 0 and default not in [REQUIRED, OPTIONAL]:
       # convenience method to used default value to derive type
       args.append(type(default))
 
@@ -414,7 +430,7 @@ class Restricted(Validator):
     return val
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-def valid(*validators):
+def valid(*validators, default = NOTSET):
   """Casts list of objects to Validator, if needed
   """
   if len(validators) == 1:
@@ -423,13 +439,13 @@ def valid(*validators):
     if isinstance(v, Validator):
       return v
 
-  return Validator(*validators)
+  return Validator(*validators, default = default)
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 def union(*validators):
   """Value must pass at least one of listed validators
   """
-  return Validator([valid(v) for v in validators], default = required)
+  return Validator([valid(v) for v in validators], default = REQUIRED)
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 def restrict(*options):
@@ -521,8 +537,9 @@ def valid_keys(
       if k_old in out:
         out = copy_once(out)
 
-        if k_new and k_new is not optional:
-          if k_new is required:
+        if k_new and OPTIONAL != k_new:
+          if REQUIRED == k_new:
+            # Use of REQUIRED indicates this is an error
             raise ValidationError(f"Use of key '{k_old}' is deprecated")
           else:
             warnings.warn(f"Use of key '{k_old}' is deprecated, replaced by '{k_new}'", DeprecationWarning)
@@ -581,7 +598,7 @@ def valid_keys(
     allow_keys.extend( [
         k_new
         for k_old, k_new in deprecate_keys
-        if k_new not in [None, optional, required ] ]
+        if k_new not in [None, OPTIONAL, REQUIRED ] ]
       if deprecate_keys else [] )
 
     allow_keys.extend( default.keys() if default else [] )
@@ -756,6 +773,7 @@ class valid_dict(Mapping):
     return self
 
   #---------------------------------------------------------------------------#
+  # pylint: disable-next=E0602
   def __init__( self, *args, **kwargs ):
     cls = type(self)
 
@@ -766,7 +784,7 @@ class valid_dict(Mapping):
 
       v = args[0]
 
-      if v in [None, optional]:
+      if v in [None, OPTIONAL]:
         args = [dict()]
       elif cls._proxy_key:
         args = [{ cls._proxy_key : v }]
@@ -780,6 +798,7 @@ class valid_dict(Mapping):
       self._p_all_keys = list()
       self._validating = False
 
+    self._p_all_keys.extend( self._allow_keys or [] )
     self._p_all_keys.extend( self._require_keys or [] )
     self._p_all_keys.extend( self._default.keys() )
 
@@ -788,7 +807,7 @@ class valid_dict(Mapping):
         self._p_all_keys.extend( [
             k_new
             for k_old, k_new in self._deprecate_keys
-            if k_new not in [None, optional, required] ] )
+            if k_new not in [None, OPTIONAL, REQUIRED] ] )
 
     if self._min_keys:
       for keys in self._min_keys:
@@ -898,13 +917,13 @@ class valid_dict(Mapping):
       pass
 
     if name != '_p_dict' and name != '_p_key_attr':
-      if name in self._p_key_attr:
-        self._p_dict[ self._p_key_attr[name] ] = val
+      if name in self._p_dict:
+        self._p_dict[ name ] = val
         self._validate()
         return
 
-      if name in self._p_dict:
-        self._p_dict[ name ] = val
+      if name in self._p_key_attr:
+        self._p_dict[ self._p_key_attr[name] ] = val
         self._validate()
         return
 
@@ -930,12 +949,12 @@ class valid_dict(Mapping):
 
     # only get mapping if base object does not have attribute
     if name != '_p_dict' and name != '_p_key_attr':
-      if name in self._p_key_attr:
-        return self._p_dict[ self._p_key_attr[name] ]
-
       if name in self._p_dict:
         return self._p_dict[ name ]
 
+      if name in self._p_key_attr:
+        return self._p_dict[ self._p_key_attr[name] ]
+
     raise AttributeError(
       f"'{type(self).__name__}' object has no key '{name}'")
 
@@ -1069,7 +1088,7 @@ def mapget(
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 def as_list( obj ):
-  if isinstance( obj, str ) or not isinstance(obj, Iterable):
+  if isinstance( obj, str ) or isinstance(obj, Mapping) or not isinstance(obj, Iterable):
     return [ obj ]
 
   return list(obj)
